#include <stdlib.h>

#include "Markov_manager.h"
#include "Markov_stride.h"

static SMarkovManager* Markov_Manager_Add(SMarkovManager* manager, SMarkov *m,int key);

/* Nothing special in this file,
 * just a declaration of a chained list and
 * a few functions to make it all work...
 * I did not comment it since it is pretty basic stuff
 */


/* global declaration */
static SMarkovManager* g_markovm = NULL;

SMarkovManager* Markov_Manager_Create()
{
	SMarkovManager* res = malloc(sizeof(*res));
	if(res==NULL)
	{
		printf("Allocation error in create Markovian Manager\n");
		exit(0); /* a little brutal but it'll get their attention */
	}
	res->m = NULL;
	res->cnt = 0;

	g_markovm = res;

	return res;
}

SMarkovManager* Markov_Manager_Add(SMarkovManager* manager, SMarkov *m,int key)
{
	SMarkov_list* tmp;
	
	manager->cnt ++;
	
	tmp = malloc(sizeof(*tmp));

	if(tmp==NULL) {
		printf("Allocation error 2 in create Markovian Manager\n");
		exit(0); /* a little brutal but it'll get their attention */
    }

	 tmp->m = m;
	 tmp->next = manager->m; 
	 tmp->key = key;
	 
	 manager->m = tmp;
	 return manager;
}

void Markov_Manager_Call(int key,void *add)
{
	SMarkov_list* tmp = g_markovm->m;
	SMarkov *markov_tmp;
	
	while(tmp)
	{
		if(tmp->key == key)
			{
			/* call the markovian model */
			tmp->m->fct(tmp->m,add);
			return; /* get out */
			}
		tmp = tmp->next;
	}
	
	/* we don't have this key */
	markov_tmp = Markov_Initialize();
    /* Call for this address */
    markov_tmp->fct(markov_tmp, add);
	
	g_markovm = Markov_Manager_Add(g_markovm,markov_tmp,key); 
}

void Markov_Manager_SetAddress(int key,void *add)
{
	SMarkov_list* tmp = g_markovm->m;
	SMarkov *markov_tmp;
	
	while(tmp)
	{
		if(tmp->key == key)
			{
			/* call the markovian model */
            Markov_SetAddress(tmp->m,add);
			return; /* get out */
			}
		tmp = tmp->next;
	}
	
	/* we don't have this key */
	markov_tmp = Markov_Initialize();
    Markov_SetAddress(markov_tmp, add);
	
	g_markovm = Markov_Manager_Add(g_markovm,markov_tmp,key); 
}

void Markov_Manager_Set(int key, int val, int type)
{
	SMarkov_list* tmp = g_markovm->m;
	SMarkov *markov_tmp;
	
	while(tmp)
	{
		if(tmp->key == key)
			{
			/* call the markovian model */
            Markov_Set(tmp->m, val, type);
			return; /* get out */
			}
		tmp = tmp->next;
	}
	
	/* we don't have this key */
	markov_tmp = Markov_Initialize();
    Markov_Set(markov_tmp, val, type);
	
	g_markovm = Markov_Manager_Add(g_markovm,markov_tmp,key); 
}

void Markov_Manager_Stop()
{
	SMarkov_list* tmp,*tmp2;
	
	if(g_markovm==NULL)
		return;

	tmp = g_markovm->m;

    //To clear the profile file
    FILE *f = fopen("Markov-profile.p", "w");
    if(f == NULL) {
        fprintf(stderr, "Error with fopen in Markov_Manager_Stop\n");
        exit(EXIT_FAILURE);
    }

	printf("////STARTING////\n");
	while(tmp)
	{
		if(tmp->m->fct != &Markov_Idle)
		{
			printf("%d\n",tmp->key);
            Markov_DisplayInfoSummarized(tmp->m);
            if(Markov_DoWeOptimize(tmp->m)) {
                fprintf(f, "%d\n", tmp->key);
            }
			tmp2 = tmp->next;
			Markov_Clear(tmp->m);
			free(tmp);
			tmp = tmp2;
		}
	}
	
    fclose(f);
	free(g_markovm), g_markovm = NULL;
}
