/*
  This code builds a framework that helps model a program's behaviour. It is programmed
  in such a way that is should seem to the user as being a black-box. Not a lot of functions
  are needed to be able to make it run.

  For any problem of installation or usage, please email me at: beyler@dpt-info.u-strasbg.fr

  Copyright (C) 2006 Beyler Jean Christophe

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include <stdio.h>
#include <stdlib.h>

#include "../Markov_manager.h"

#define O 3
#define N 2
#define M 100000

int main(void)
{
int i;

//Initialize memory
Markov_Manager_Create();

//Setting variables
printf("Setting variables\n");
Markov_Manager_Set(0,5,M_CONSDEPTH); //Set depth of construction
Markov_Manager_Set(0,50,M_ERRMAX);   //Set maximum errors before flushing
Markov_Manager_Set(0,30,M_CONSTTL);  //Set number of calls used for the construction phase
Markov_Manager_Set(0,1,M_PREFDIST);  //Change the prefetching distance

//Init table
for(i=0;i<M;i++)
   {
   unsigned char *rdm = (unsigned char*) rand();
   
   Markov_Manager_SetAddress(0, rdm);
   Markov_Manager_Call(0, rdm);
   Markov_Manager_Call(0, rdm+240);
   Markov_Manager_Call(0, rdm+123);
   Markov_Manager_Call(0, rdm+432);
   }
      
printf("Finished simulation\n");
/* Free memory */
Markov_Manager_Stop();

return EXIT_SUCCESS;
}
